@@ -265,6 +265,10 @@ DC_Exit( int status, const char *shutdown_program )
 		XMLObj = NULL;
 	}
 
+#ifdef LINUX
+		// Remove any keys stored in the kernel (for ecryptfs)
+	FilesystemRemap::EcryptfsUnlinkKeys();
+#endif
 
 		// See if this daemon wants to be restarted (true by
 		// default).  If so, use the given status.  Otherwise, use the
@@ -672,7 +676,8 @@ handle_dynamic_dirs()
 	}
 	int mypid = daemonCore->getpid();
 	char buf[256];
-	sprintf( buf, "%s-%d", get_local_ipaddr().to_ip_string().Value(), mypid );
+	// TODO: Picking IPv4 arbitrarily.
+	sprintf( buf, "%s-%d", get_local_ipaddr(CP_IPV4).to_ip_string().Value(), mypid );
 
 	set_dynamic_dir( "LOG", buf );
 	set_dynamic_dir( "SPOOL", buf );
@@ -1034,6 +1039,7 @@ handle_fetch_log( Service *, int cmd, ReliSock *stream )
 	}
 
 	char *pname = (char*)malloc (strlen(name) + 5);
+	ASSERT(pname);
 	char *ext = strchr(name,'.');
 
 	//If there is a dot in the name, it is of the form "<SUBSYS>.<ext>"
