@@ -22,7 +22,7 @@
 #include "condor_attributes.h"
 #include "condor_debug.h"
 #include "condor_string.h"	// for strnewp and friends
-#include "../condor_daemon_core.V6/condor_daemon_core.h"
+#include "condor_daemon_core.h"
 #include "basename.h"
 #include "nullfile.h"
 #include "filename_tools.h"
@@ -36,6 +36,8 @@
 #include "ec2job.h"
 #include "condor_config.h"
 
+#include <algorithm>
+
 using namespace std;
 
 #define GM_INIT							0
@@ -201,18 +203,50 @@ EC2Job::EC2Job( ClassAd *classad ) :
 	if( m_failure_injection == NULL ) { m_failure_injection = ""; }
 	dprintf( D_FULLDEBUG, "GM_FAILURE_INJECTION = %s\n", m_failure_injection );
 
-	// lookup the elastic IP
 	jobAd->LookupString( ATTR_EC2_ELASTIC_IP, m_elastic_ip );
-
 	jobAd->LookupString( ATTR_EC2_EBS_VOLUMES, m_ebs_volumes );
-
-	// lookup the elastic IP
 	jobAd->LookupString( ATTR_EC2_AVAILABILITY_ZONE, m_availability_zone );
-
 	jobAd->LookupString( ATTR_EC2_VPC_SUBNET, m_vpc_subnet );
-
 	jobAd->LookupString( ATTR_EC2_VPC_IP, m_vpc_ip );
+	jobAd->LookupString( ATTR_EC2_BLOCK_DEVICE_MAPPING, m_block_device_mapping );
+	jobAd->LookupString( ATTR_EC2_IAM_PROFILE_ARN, m_iam_profile_arn );
+	jobAd->LookupString( ATTR_EC2_IAM_PROFILE_NAME, m_iam_profile_name );
 
+	// There's no reason not to build the list of group names here, as well.
+	{
+		std::string securityIDs;
+		m_group_ids = new StringList();
+		jobAd->LookupString( ATTR_EC2_SECURITY_IDS, securityIDs );
+		if( ! securityIDs.empty() ) {
+			m_group_ids->initializeFromString( securityIDs.c_str() );
+		}
+	}
+
+	{
+		std::string paramNames;
+		m_parameters_and_values = new StringList();
+		jobAd->LookupString( ATTR_EC2_PARAM_NAMES, paramNames );
+		if( ! paramNames.empty() ) {
+			StringList paramNameList( paramNames.c_str() );
+
+			const char * paramName = NULL;
+			paramNameList.rewind();
+			while( (paramName = paramNameList.next()) ) {
+				std::string paramValue;
+				std::string jobAdName = paramName;
+				std::replace( jobAdName.begin(), jobAdName.end(), '.', '_' );
+				formatstr( jobAdName, "%s_%s", ATTR_EC2_PARAM_PREFIX, jobAdName.c_str() );
+
+				jobAd->LookupString( jobAdName.c_str(), paramValue );
+				if( paramValue.empty() ) {
+					dprintf( D_ALWAYS, "EC2 parameter '%s' had no corresponding value, ignoring.\n", paramName );
+					continue;
+				}
+				m_parameters_and_values->append( paramName );
+				m_parameters_and_values->append( paramValue.c_str() );
+			}
+		}
+	}
 
 	// if user assigns both user_data and user_data_file, the two will
 	// be concatenated by the gahp
@@ -460,6 +494,7 @@ void EC2Job::doEvaluateState()
 
 		reevaluate_state = false;
 		old_gm_state = gmState;
+		ASSERT ( gahp != NULL || gmState == GM_HOLD || gmState == GM_DELETE );
 
 		switch ( gmState )
 		{
@@ -763,7 +798,12 @@ void EC2Job::doEvaluateState()
 											 m_vpc_subnet,
 											 m_vpc_ip,
 											 m_client_token,
+											 m_block_device_mapping,
+											 m_iam_profile_arn,
+											 m_iam_profile_name,
 											 *m_group_names,
+											 *m_group_ids,
+											 *m_parameters_and_values,
 											 instance_id,
 											 gahp_error_code);
 
