@@ -81,6 +81,7 @@
 #define ATTR_CKPT_LAST_READ  "CkptLastRead"
 #define ATTR_CKPT_OPSYS  "CkptOpSys"
 #define ATTR_PAIRED_CLAIM_ID  "PairedClaimId"
+#define ATTR_CHECKPOINT_SIG  "CheckpointSig"
 #define ATTR_CHILD_CLAIM_IDS "ChildClaimIds"
 #define ATTR_CLAIM_ID  "ClaimId"
 #define ATTR_CLAIM_IDS  "ClaimIds"
@@ -170,6 +171,8 @@
 #define ATTR_DESTINATION  "Destination"
 #define ATTR_DISK  "Disk"
 #define ATTR_DISK_USAGE  "DiskUsage"
+#define ATTR_DOCKER_IMAGE "DockerImage"
+#define ATTR_DOCKER_VERSION  "DockerVersion"
 #define ATTR_EMAIL_ATTRIBUTES  "EmailAttributes"
 #define ATTR_ENTERED_CURRENT_ACTIVITY  "EnteredCurrentActivity"
 #define ATTR_ENTERED_CURRENT_STATE  "EnteredCurrentState"
@@ -241,6 +244,8 @@
 // END ckireyev myproxy
 #define ATTR_HARDWARE_ADDRESS  "HardwareAddress"
 #define ATTR_HAS_CHECKPOINTING  "HasCheckpointing"
+#define ATTR_HAS_DOCKER  "HasDocker"
+#define ATTR_HAS_ENCRYPT_EXECUTE_DIRECTORY "HasEncryptExecuteDirectory"
 #define ATTR_HAS_FILE_TRANSFER  "HasFileTransfer"
 #define ATTR_HAS_FILE_TRANSFER_PLUGIN_METHODS  "HasFileTransferPluginMethods"
 #define ATTR_HAS_PER_FILE_ENCRYPTION  "HasPerFileEncryption"
@@ -304,6 +309,7 @@
 #define ATTR_JOB_CURRENT_START_TRANSFER_OUTPUT_DATE  "JobCurrentStartTransferOutputDate"
 #define ATTR_CUMULATIVE_TRANSFER_TIME  "CumulativeTransferTime"
 #define ATTR_JOB_DURATION  "JobDuration"
+#define ATTR_ENCRYPT_EXECUTE_DIRECTORY "EncryptExecuteDirectory"
 #define ATTR_JOB_ENVIRONMENT1  "Env"
 #define ATTR_JOB_ENVIRONMENT1_DELIM  "EnvDelim"
 #define ATTR_JOB_ENVIRONMENT2  "Environment"
@@ -430,6 +436,7 @@
 #define ATTR_PARENT_SLOT_ID "ParentSlotId"
 #define ATTR_RETIREMENT_TIME_REMAINING "RetirementTimeRemaining"
 #define ATTR_MAX_JOBS_RUNNING  "MaxJobsRunning"
+#define ATTR_CURB_MATCHMAKING "CurbMatchmaking"
 #define ATTR_MEMORY  "Memory"
 #define ATTR_MEMORY_USAGE  "MemoryUsage"
 #define ATTR_DETECTED_MEMORY  "DetectedMemory"
@@ -451,7 +458,6 @@
 #define ATTR_NODE  "Node"
 #define ATTR_NORDUGRID_RSL  "NordugridRSL"
 #define ATTR_NOTIFY_USER  "NotifyUser"
-#define ATTR_NOTIFY_JOB_SCHEDULER  "NotifyJobScheduler"
 #define ATTR_NT_DOMAIN  "NTDomain"
 #define ATTR_TRANSFER_QUEUE_USER_EXPR "TransferQueueUserExpr"
 //extern const char ATTR_WINDOWS_VERSION [];
@@ -467,6 +473,7 @@
 #define ATTR_NUM_DYNAMIC_SLOTS  "NumDynamicSlots"
 #define ATTR_NUM_GLOBUS_SUBMITS  "NumGlobusSubmits"
 #define ATTR_NUM_MATCHES  "NumJobMatches"
+#define ATTR_LIMIT_RESULTS "LimitResults"
 #define ATTR_NUM_HOPS_TO_SUBMIT_MACHINE  "NumHopsToSubmitMachine"
 #define ATTR_NUM_HOPS_TO_LAST_CKPT_SERVER  "NumHopsToLastCkptServer"
 #define ATTR_NUM_HOPS_TO_CKPT_SERVER  "NumHopsToCkptServer"
@@ -508,6 +515,7 @@
 #define ATTR_PREV_RECV_ESTIMATE  "PrevRecvEstimate"
 #define ATTR_PRIO  "Prio"
 #define ATTR_PROC_ID  "ProcId"
+#define ATTR_PSLOT_ROLLUP_INFORMATION "PslotRollupInformation"
 #define ATTR_SUB_PROC_ID  "SubProcId"
 #define ATTR_PRIVATE_NETWORK_NAME  "PrivateNetworkName"
 #define ATTR_Q_DATE  "QDate"
@@ -536,6 +544,8 @@
 #define ATTR_REQUEUE_REASON  "RequeueReason"
 #define ATTR_REQUIREMENTS  "Requirements"
 #define ATTR_RESOURCE_REQUEST_COUNT "_condor_RESOURCE_COUNT"  // used in resource request ad
+#define ATTR_RESOURCE_REQUEST_CLUSTER "_condor_RESOURCE_CLUSTER"
+#define ATTR_RESOURCE_REQUEST_PROC "_condor_RESOURCE_PROC"
 #define ATTR_SLOT_TYPE  "SlotType"
 #define ATTR_SLOT_TYPE_ID  "SlotTypeID"
 #define ATTR_SLOT_WEIGHT  "SlotWeight"
@@ -547,6 +557,8 @@
 #define ATTR_RUN_BENCHMARKS  "RunBenchmarks"
 #define ATTR_SHADOW_IP_ADDR  "ShadowIpAddr"
 #define ATTR_MY_ADDRESS  "MyAddress"
+#define ATTR_NUM_JOB_STARTS_DELAYED  "NumJobStartsDelayed"
+#define ATTR_NUM_PENDING_CLAIMS  "NumPendingClaims"
 #define ATTR_SCHEDD_SWAP_EXHAUSTED  "ScheddSwapExhausted"
 #define ATTR_SCHEDD_INTERVAL  "ScheddInterval"
 #define ATTR_SCHEDD_IP_ADDR  "ScheddIpAddr"
@@ -568,6 +580,7 @@
 #define ATTR_STARTD_IP_ADDR  "StartdIpAddr"
 #define ATTR_STARTD_PRINCIPAL  "StartdPrincipal"
 #define ATTR_STARTD_SENDS_ALIVES  "StartdSendsAlives"
+#define ATTR_STARTER_HANDLES_ALIVES "_condor_StartdHandlesAlives"
 #define ATTR_STATE  "State"
 #define ATTR_STARTER_IP_ADDR  "StarterIpAddr"
 #define ATTR_STARTER_ABILITY_LIST  "StarterAbilityList"
@@ -695,6 +708,8 @@
 #define ATTR_WANT_DELAYED_UPDATES "WantDelayedUpdates"
 #define ATTR_WANT_MATCH_DIAGNOSTICS  "WantMatchDiagnostics"
 #define ATTR_WANT_PARALLEL_SCHEDULING_GROUPS  "WantParallelSchedulingGroups"
+#define ATTR_WANT_CHECKPOINT_SIGNAL  "WantCheckpointSignal"
+#define ATTR_WANT_PSLOT_PREEMPTION  "WantPslotPreemption"
 #define ATTR_WANT_REMOTE_SYSCALLS  "WantRemoteSyscalls"
 #define ATTR_WANT_REMOTE_IO  "WantRemoteIO"
 #define ATTR_WANT_SCHEDD_COMPLETION_VISA  "WantCompletionVisaFromSchedD"
@@ -703,6 +718,7 @@
 #define ATTR_WANT_LAST_CKPT_SERVER_NET_STATS  "WantLastCkptServerNetStats"
 #define ATTR_WANT_CKPT_SERVER_NET_STATS  "WantCkptServerNetStats"
 #define ATTR_WANT_AD_REVAULATE  "WantAdRevaluate"
+#define ATTR_WANT_DOCKER  "WantDocker"
 #define ATTR_COLLECTOR_IP_ADDR  "CollectorIpAddr"
 #define ATTR_NEGOTIATOR_IP_ADDR  "NegotiatorIpAddr"
 #define ATTR_CREDD_IP_ADDR  "CredDIpAddr"
@@ -812,6 +828,7 @@
 extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_SEC_TRIED_AUTHENTICATION  "TriedAuthentication"
 #define ATTR_SEC_AUTHORIZATION_SUCCEEDED  "AuthorizationSucceeded"
+#define ATTR_SEC_RETURN_CODE  "ReturnCode"
 
 #define ATTR_MULTIPLE_TASKS_PER_PVMD  "MultipleTasksPerPvmd"
 
@@ -892,6 +909,7 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_EC2_KEY_PAIR  "EC2KeyPair"
 #define ATTR_EC2_KEY_PAIR_FILE  "EC2KeyPairFile"
 #define ATTR_EC2_SECURITY_GROUPS  "EC2SecurityGroups"
+#define ATTR_EC2_SECURITY_IDS  "EC2SecurityIDs"
 #define ATTR_EC2_USER_DATA  "EC2UserData"
 #define ATTR_EC2_USER_DATA_FILE  "EC2UserDataFile"
 #define ATTR_EC2_REMOTE_VM_NAME  "EC2RemoteVirtualMachineName"
@@ -908,12 +926,18 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_EC2_SPOT_REQUEST_ID  "EC2SpotRequestID"
 #define ATTR_EC2_STATUS_REASON_CODE  "EC2StatusReasonCode"
 #define ATTR_EC2_SERVER_TYPE  "EC2ServerType"
+#define ATTR_EC2_BLOCK_DEVICE_MAPPING "EC2BlockDeviceMapping"
+#define ATTR_EC2_PARAM_NAMES  "EC2ParamNames"
+#define ATTR_EC2_PARAM_PREFIX  "EC2Param"
+#define ATTR_EC2_IAM_PROFILE_ARN  "EC2IamProfileArn"
+#define ATTR_EC2_IAM_PROFILE_NAME  "EC2IamProfileName"
 
 //************* End of changes for EC2 Jobs *****************//
 
 #define ATTR_REQUEST_CPUS  "RequestCpus"
 #define ATTR_REQUEST_MEMORY  "RequestMemory"
 #define ATTR_REQUEST_DISK  "RequestDisk"
+#define ATTR_REQUEST_VIRTUAL_MEMORY  "RequestVirtualMemory"
 
 // machine resource prefixes
 #define ATTR_REQUEST_PREFIX  "Request"
@@ -1039,6 +1063,8 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_PROJECTION  "Projection"
 #define ATTR_LAST_DRAIN_START_TIME  "LastDrainStartTime"
 
+#define ATTR_SHARED_PORT_COMMAND_SINFULS "SharedPortCommandSinfuls"
+
 // temporary attributes for raw utsname info
 #define ATTR_UTSNAME_SYSNAME  "UtsnameSysname"
 #define ATTR_UTSNAME_NODENAME  "UtsnameNodename"
