@@ -22,7 +22,7 @@
 #include "JobRouterHookMgr.h"
 #include "status_string.h"
 #include "JobRouter.h"
-#include "set_user_from_ad.h"
+#include "set_user_priv_from_ad.h"
 #include "Scheduler.h"
 #include "submit_job.h"
 
@@ -37,6 +37,10 @@ SimpleList<HOOK_RUN_INFO*> JobRouterHookMgr::m_job_hook_list;
 
 JobRouterHookMgr::JobRouterHookMgr()
 	: HookClientMgr(),
+	  m_warn_cleanup(true),
+	  m_warn_update(true),
+	  m_warn_translate(true),
+	  m_warn_exit(true),
 	  NUM_HOOKS(4),
 	  UNDEFINED("UNDEFINED"),
 	  m_hook_paths(MyStringHash)
@@ -111,12 +115,16 @@ JobRouterHookMgr::reconfig()
 		free(m_default_hook_keyword);
 	}
 	m_default_hook_keyword = param("JOB_ROUTER_HOOK_KEYWORD");
+	m_warn_translate = true;
+	m_warn_exit = true;
+	m_warn_update = true;
+	m_warn_cleanup = true;
 	return true;
 }
 
 
 char*
-JobRouterHookMgr::getHookPath(HookType hook_type, classad::ClassAd ad)
+JobRouterHookMgr::getHookPath(HookType hook_type, const classad::ClassAd &ad)
 {
 	std::string keyword;
 	char** paths;
@@ -166,7 +174,7 @@ JobRouterHookMgr::getHookPath(HookType hook_type, classad::ClassAd ad)
 
 
 std::string
-JobRouterHookMgr::getHookKeyword(classad::ClassAd ad)
+JobRouterHookMgr::getHookKeyword(const classad::ClassAd &ad)
 {
 	std::string hook_keyword;
 
@@ -189,7 +197,10 @@ JobRouterHookMgr::hookTranslateJob(RoutedJob* r_job, std::string &route_info)
 	if (NULL == hook_translate)
 	{
 		// hook not defined, which is ok
-		dprintf(D_FULLDEBUG, "HOOK_TRANSLATE_JOB not configured.\n");
+		if (m_warn_translate) {
+			dprintf(D_FULLDEBUG, "HOOK_TRANSLATE_JOB not configured.\n");
+			m_warn_translate = false;
+		}
 		return 0;
 	}
 
@@ -220,7 +231,7 @@ JobRouterHookMgr::hookTranslateJob(RoutedJob* r_job, std::string &route_info)
 		return -1;
 	}
 
-	set_user_from_ad(r_job->src_ad);
+	set_user_priv_from_ad(r_job->src_ad);
 	if (0 == spawn(translate_client, NULL, &hook_stdin, PRIV_USER_FINAL))
 	{
 		dprintf(D_ALWAYS|D_FAILURE,
@@ -254,7 +265,10 @@ JobRouterHookMgr::hookUpdateJobInfo(RoutedJob* r_job)
 	if (NULL == hook_update_job_info)
 	{
 		// hook not defined
-		dprintf(D_FULLDEBUG, "HOOK_UPDATE_JOB_INFO not configured.\n");
+		if (m_warn_update) {
+			dprintf(D_FULLDEBUG, "HOOK_UPDATE_JOB_INFO not configured.\n");
+			m_warn_update = false;
+		}
 		return 0;
 	}
 
@@ -284,7 +298,7 @@ JobRouterHookMgr::hookUpdateJobInfo(RoutedJob* r_job)
 		return -1;
 	}
 
-	set_user_from_ad(r_job->src_ad);
+	set_user_priv_from_ad(r_job->src_ad);
 	if (0 == spawn(status_client, NULL, &hook_stdin, PRIV_USER_FINAL))
 	{
 		dprintf(D_ALWAYS|D_FAILURE,
@@ -319,7 +333,10 @@ JobRouterHookMgr::hookJobExit(RoutedJob* r_job)
 	if (NULL == hook_job_exit)
 	{
 		// hook not defined
-		dprintf(D_FULLDEBUG, "HOOK_JOB_EXIT not configured.\n");
+		if (m_warn_exit) {
+			dprintf(D_FULLDEBUG, "HOOK_JOB_EXIT not configured.\n");
+			m_warn_exit = false;
+		}
 		return 0;
 	}
 
@@ -352,7 +369,7 @@ JobRouterHookMgr::hookJobExit(RoutedJob* r_job)
 		return -1;
 	}
 
-	set_user_from_ad(r_job->src_ad);
+	set_user_priv_from_ad(r_job->src_ad);
 	if (0 == spawn(exit_client, NULL, &hook_stdin, PRIV_USER_FINAL))
 	{
 		dprintf(D_ALWAYS|D_FAILURE,
@@ -386,7 +403,10 @@ JobRouterHookMgr::hookJobCleanup(RoutedJob* r_job)
 	if (NULL == hook_cleanup)
 	{
 		// hook not defined
-		dprintf(D_FULLDEBUG, "HOOK_JOB_CLEANUP not configured.\n");
+		if (m_warn_cleanup) {
+			dprintf(D_FULLDEBUG, "HOOK_JOB_CLEANUP not configured.\n");
+			m_warn_cleanup = false;
+		}
 		return 0;
 	}
 
@@ -421,7 +441,7 @@ JobRouterHookMgr::hookJobCleanup(RoutedJob* r_job)
 		return -1;
 	}
 
-	set_user_from_ad(r_job->src_ad);
+	set_user_priv_from_ad(r_job->src_ad);
 	if (0 == spawn(cleanup_client, NULL, &hook_stdin, PRIV_USER_FINAL))
 	{
 		dprintf(D_ALWAYS|D_FAILURE,
@@ -458,6 +478,7 @@ JobRouterHookMgr::addKnownHook(const char* key, HookType hook)
 	ASSERT( hook_info != NULL );
 	hook_info->hook_type = hook;
 	hook_info->key = strdup(key);
+	ASSERT( hook_info->key );
 	return(m_job_hook_list.Append(hook_info));
 }
 
