@@ -34,7 +34,8 @@ class XInterface
     XInterface(int id);
     ~XInterface();
 
-    
+    void SetBumpCheck(int delta_move, int delta_time);
+
     bool CheckActivity();
   private:
     bool ProcessEvents();
@@ -56,6 +57,9 @@ class XInterface
     int          _pointer_prev_x;
     int          _pointer_prev_y;
 
+    int          _small_move_delta;
+    int          _bump_check_after_idle_time_sec;
+
     bool        _tried_root;
     bool        _tried_utmp;
     int         _daemon_core_timer;
