@@ -26,6 +26,12 @@
 #include "condor_config.h"
 #include "Regex.h"
 #include "classad/classadCache.h"
+#include "env.h"
+#include "condor_arglist.h"
+
+#if defined(HAVE_DLOPEN)
+#include <dlfcn.h>
+#endif
 
 using namespace std;
 
@@ -48,6 +54,9 @@ namespace compat_classad {
 
 static StringList ClassAdUserLibs;
 
+static void registerClassadFunctions();
+static void classad_debug_dprintf(const char *s);
+
 bool ClassAd::m_initConfig = false;
 bool ClassAd::m_strictEvaluation = false;
 
@@ -55,7 +64,7 @@ void ClassAd::
 Reconfig()
 {
 	m_strictEvaluation = param_boolean( "STRICT_CLASSAD_EVALUATION", false );
-	classad::_useOldClassAdSemantics = !m_strictEvaluation;
+	classad::SetOldClassAdSemantics( !m_strictEvaluation );
 
 	classad::ClassAdSetExpressionCaching( param_boolean( "ENABLE_CLASSAD_CACHING", false ) );
 
@@ -76,21 +85,42 @@ Reconfig()
 			}
 		}
 	}
-}
 
-void getTheMyRef( classad::ClassAd *ad )
-{
-	if ( !ClassAd::m_strictEvaluation ) {
-		ExprTree * pExpr=classad::AttributeReference::MakeAttributeReference( NULL, "self" );
-		ad->Insert( "my", pExpr );
+	char *user_python_char = param("CLASSAD_USER_PYTHON_MODULES");
+	if (user_python_char)
+	{
+		std::string user_python(user_python_char);
+		free(user_python_char); user_python_char = NULL;
+		char *loc_char = param("CLASSAD_USER_PYTHON_LIB");
+		if (loc_char && !ClassAdUserLibs.contains(loc_char))
+		{
+			std::string loc(loc_char);
+			if (classad::FunctionCall::RegisterSharedLibraryFunctions(loc.c_str()))
+			{
+				ClassAdUserLibs.append(loc.c_str());
+#if defined(HAVE_DLOPEN)
+				void *dl_hdl = dlopen(loc.c_str(), RTLD_LAZY);
+				if (dl_hdl) // Not warning on failure as the RegisterSharedLibraryFunctions should have done that.
+				{
+					void (*registerfn)(void) = (void (*)(void))dlsym(dl_hdl, "Register");
+					if (registerfn) {registerfn();}
+					dlclose(dl_hdl);
+				}
+#endif
+			}
+			else
+			{
+				dprintf(D_ALWAYS, "Failed to load ClassAd user python library %s: %s\n",
+					loc.c_str(), classad::CondorErrMsg.c_str());
+			}
+		}
+		if (loc_char) {free(loc_char);}
 	}
-}
-
-void releaseTheMyRef( classad::ClassAd *ad )
-{
-	if ( !ClassAd::m_strictEvaluation ) {
-		ad->Delete("my");
-		ad->MarkAttributeClean( "my" );
+	if (!m_initConfig)
+	{
+		registerClassadFunctions();
+		classad::ExprTree::set_user_debug_function(classad_debug_dprintf);
+		m_initConfig = true;
 	}
 }
 
@@ -520,7 +550,7 @@ bool splitArb_func( const char * /*name*/,
 		return true;
 	}
 
-	classad::ExprList *lst = new classad::ExprList();
+	classad_shared_ptr<classad::ExprList> lst( new classad::ExprList() );
 	ASSERT(lst);
 
 	// walk the input string, splitting at each instance of a separator
@@ -559,10 +589,452 @@ bool splitArb_func( const char * /*name*/,
 	return true;
 }
 
+
+static void
+problemExpression(const std::string &msg, classad::ExprTree *problem, classad::Value &result)
+{
+	result.SetErrorValue();
+	classad::ClassAdUnParser up;
+	std::string problem_str;
+	up.Unparse(problem_str, problem);
+	std::stringstream ss;
+	ss << msg << "  Problem expression: " << problem_str;
+	classad::CondorErrMsg = ss.str();
+}
+
+
+static bool
+ArgsToList( const char * name,
+	const classad::ArgumentList &arguments,
+	classad::EvalState &state,
+	classad::Value &result )
+{
+	if ((arguments.size() != 1) && (arguments.size() != 2))
+	{
+		std::stringstream ss;
+		result.SetErrorValue();
+		ss << "Invalid number of arguments passed to " << name << "; one string argument expected.";
+		classad::CondorErrMsg = ss.str();
+		return true;
+	}
+	int vers = 2;
+	if (arguments.size() == 2)
+	{
+		classad::Value val;
+		if (!arguments[1]->Evaluate(state, val))
+		{
+			problemExpression("Unable to evaluate second argument.", arguments[1], result);
+			return false;
+		}
+		if (!val.IsIntegerValue(vers))
+		{
+			problemExpression("Unable to evaluate second argument to integer.", arguments[1], result);
+			return true;
+		}
+		if ((vers != 1) && (vers != 2))
+		{
+			std::stringstream ss;
+			ss << "Valid values for version are 1 or 2.  Passed expression evaluates to " << vers << ".";
+			problemExpression(ss.str(), arguments[1], result);
+			return true;
+		}
+	}
+	classad::Value val;
+	if (!arguments[0]->Evaluate(state, val))
+	{
+		problemExpression("Unable to evaluate first argument.", arguments[0], result);
+		return false;
+	}
+	std::string args;
+	if (!val.IsStringValue(args))
+	{
+		problemExpression("Unable to evaluate first argument to string.", arguments[0], result);
+		return true;
+	}
+	ArgList arg_list;
+	MyString error_msg;
+	if ((vers == 1) && !arg_list.AppendArgsV1Raw(args.c_str(), &error_msg))
+	{
+		std::stringstream ss;
+		ss << "Error when parsing argument to arg V1: " << error_msg.Value();
+		problemExpression(ss.str(), arguments[0], result);
+		return true;
+	}
+	else if ((vers == 2) && !arg_list.AppendArgsV2Raw(args.c_str(), &error_msg))
+	{
+		std::stringstream ss;
+		ss << "Error when parsing argument to arg V2: " << error_msg.Value();
+		problemExpression(ss.str(), arguments[0], result);
+		return true;
+	}
+	std::vector<classad::ExprTree*> list_exprs;
+
+	for (int idx=0; idx<arg_list.Count(); idx++)
+	{
+		classad::Value string_val;
+		string_val.SetStringValue(arg_list.GetArg(idx));
+		classad::ExprTree *expr = classad::Literal::MakeLiteral(string_val);
+		if (!expr)
+		{
+			for (std::vector<classad::ExprTree*>::iterator it = list_exprs.begin(); it != list_exprs.end(); it++)
+			{
+				if (*it) {delete *it; *it=NULL;}
+			}
+			classad::CondorErrMsg = "Unable to create string expression.";
+			result.SetErrorValue();
+			return false;
+		}
+		list_exprs.push_back(expr);
+	}
+	classad_shared_ptr<classad::ExprList> result_list(classad::ExprList::MakeExprList(list_exprs));
+	if (!result_list.get())
+	{
+		for (std::vector<classad::ExprTree*>::iterator it = list_exprs.begin(); it != list_exprs.end(); it++)
+		{
+			if (*it) {delete *it; *it=NULL;}
+		}
+		classad::CondorErrMsg = "Unable to create expression list.";
+		result.SetErrorValue();
+		return false;
+	}
+	result.SetListValue(result_list);
+	return true;
+}
+
+
+static bool
+ListToArgs(const char * name,
+	const classad::ArgumentList &arguments,
+	classad::EvalState &state,
+	classad::Value &result)
+{
+	if ((arguments.size() != 1) && (arguments.size() != 2))
+	{
+		std::stringstream ss;
+		result.SetErrorValue();
+		ss << "Invalid number of arguments passed to " << name << "; one list argument expected.";
+		classad::CondorErrMsg = ss.str();
+		return true;
+	}
+	int vers = 2;
+	if (arguments.size() == 2)
+	{
+		classad::Value val;
+		if (!arguments[1]->Evaluate(state, val))
+		{
+			problemExpression("Unable to evaluate second argument.", arguments[1], result);
+			return false;
+		}
+		if (!val.IsIntegerValue(vers))
+		{
+			problemExpression("Unable to evaluate second argument to integer.", arguments[1], result);
+			return true;
+		}
+		if ((vers != 1) && (vers != 2))
+		{
+			std::stringstream ss;
+			ss << "Valid values for version are 1 or 2.  Passed expression evaluates to " << vers << ".";
+			problemExpression(ss.str(), arguments[1], result);
+			return true;
+		}
+	}
+	classad::Value val;
+	if (!arguments[0]->Evaluate(state, val))
+	{
+		problemExpression("Unable to evaluate first argument.", arguments[0], result);
+		return false;
+	}
+	classad_shared_ptr<classad::ExprList> args;
+	if (!val.IsSListValue(args))
+	{
+		problemExpression("Unable to evaluate first argument to list.", arguments[0], result);
+		return true;
+	}
+	ArgList arg_list;
+	size_t idx=0;
+	for (classad::ExprList::const_iterator it=args->begin(); it!=args->end(); it++, idx++)
+	{
+		classad::Value value;
+		if (!(*it)->Evaluate(state, value))
+		{
+			std::stringstream ss;
+			ss << "Unable to evaluate list entry " << idx << ".";
+			problemExpression(ss.str(), *it, result);
+			return false;
+		}
+		std::string tmp_str;
+		if (!value.IsStringValue(tmp_str))
+		{
+			std::stringstream ss;
+			ss << "Entry " << idx << " did not evaluate to a string.";
+			problemExpression(ss.str(), *it, result);
+			return true;
+		}
+		arg_list.AppendArg(tmp_str.c_str());
+	}
+	MyString error_msg, result_mystr;
+	if ((vers == 1) && !arg_list.GetArgsStringV1Raw(&result_mystr, &error_msg))
+	{
+		std::stringstream ss;
+		ss << "Error when parsing argument to arg V1: " << error_msg.Value();
+		problemExpression(ss.str(), arguments[0], result);
+		return true;
+	}
+	else if ((vers == 2) && !arg_list.GetArgsStringV2Raw(&result_mystr, &error_msg))
+	{
+		std::stringstream ss;
+		ss << "Error when parsing argument to arg V2: " << error_msg.Value();
+		problemExpression(ss.str(), arguments[0], result);
+		return true;
+	}
+	result.SetStringValue(result_mystr.Value());
+	return true;
+}
+
+
+static bool
+EnvironmentV1ToV2(const char * name,
+	const classad::ArgumentList &arguments,
+	classad::EvalState &state,
+	classad::Value &result)
+{
+	if (arguments.size() != 1)
+	{
+		std::stringstream ss;
+		result.SetErrorValue();
+		ss << "Invalid number of arguments passed to " << name << "; one string argument expected.";
+		classad::CondorErrMsg = ss.str();
+		return true;
+	}
+	classad::Value val;
+	if (!arguments[0]->Evaluate(state, val))
+	{
+		problemExpression("Unable to evaluate first argument.", arguments[0], result);
+		return false;
+	}
+		// By returning undefined if the input is undefined, the caller doesn't need an
+		// ifThenElse statement to see if the V1 environment is used in the first place.
+		// For example, we can do this:
+		//   mergeEnvironment(envV1toV2(Env), Environment, "FOO=BAR");
+		// without worrying about whether Env or Environment is already defined.
+	if (val.IsUndefinedValue())
+	{
+		result.SetUndefinedValue();
+		return true;
+	}
+	std::string args;
+	if (!val.IsStringValue(args))
+	{
+		problemExpression("Unable to evaluate first argument to string.", arguments[0], result);
+		return true;
+	}
+	Env env;
+	MyString error_msg;
+	if (!env.MergeFromV1Raw(args.c_str(), &error_msg))
+	{
+		std::stringstream ss;
+		ss << "Error when parsing argument to environment V1: " << error_msg.Value();
+		problemExpression(ss.str(), arguments[0], result);
+		return true;
+	}
+	MyString result_mystr;
+	env.getDelimitedStringV2Raw(&result_mystr, NULL);
+	result.SetStringValue(result_mystr.Value());
+	return true;
+}
+
+
+static bool
+MergeEnvironment(const char * /*name*/,
+	const classad::ArgumentList &arguments,
+	classad::EvalState &state,
+	classad::Value &result)
+{
+	Env env;
+	size_t idx = 0;
+	for (classad::ArgumentList::const_iterator it=arguments.begin(); it!=arguments.end(); it++, idx++)
+	{
+		classad::Value val;
+		if (!(*it)->Evaluate(state, val))
+		{
+			std::stringstream ss;
+			ss << "Unable to evaluate argument " << idx << ".";
+			problemExpression(ss.str(), *it, result);
+			return false;
+		}
+			// Skip over undefined values; this makes it more natural
+			// to merge environments for a condor classad where the
+			// user may or may not have specified an environment string.
+		if (val.IsUndefinedValue())
+		{
+			continue;
+		}
+		std::string env_str;
+		if (!val.IsStringValue(env_str))
+		{
+			std::stringstream ss;
+			ss << "Unable to evaluate argument " << idx << ".";
+			problemExpression(ss.str(), *it, result);
+			return true;
+		}
+		MyString error_msg;
+		if (!env.MergeFromV2Raw(env_str.c_str(), &error_msg))
+		{
+			std::stringstream ss;
+			ss << "Argument " << idx << " cannot be parsed as environment string.";
+			problemExpression(ss.str(), *it, result);
+			return true;
+		}
+	}
+	MyString result_mystr;
+	env.getDelimitedStringV2Raw(&result_mystr, NULL);
+	result.SetStringValue(result_mystr.Value());
+	return true;
+}
+
+
+static bool
+return_home_result(const std::string &default_home,
+	const std::string &error_msg,
+	classad::Value    &result,
+	bool isError)
+{
+	if (default_home.size())
+	{
+		result.SetStringValue(default_home);
+	}
+	else
+	{
+		if (isError) {result.SetErrorValue();} else {result.SetUndefinedValue();}
+		classad::CondorErrMsg = error_msg;
+	}
+	return true;
+}
+
+/**
+ * Implementation of the userHome function.
+ *
+ * This returns the home directory of a given username as configured on the system.
+ * The home directory is determined using the getpwdnam call.
+ *
+ * Arguments:
+ *   - name.  The username to lookup.
+ *   - default (optional).  The default value to return if no user home exists.
+ *
+ * Returns:
+ *   - ERROR on Windows platforms.
+ *   - ERROR if invoked with the incorrect number of arguments.
+ *   - otherwise, `default` if default is given AND the value of `default` evaluates to a non-empty string AND:
+ *     - the name cannot be evaluated to a string/UNDEFINED, OR
+ *     - the user does not exist on the system, OR
+ *     - the user does not have a home directory.
+ *   - otherwise, ERROR if the first argument cannot be evaluated or does not evaluate to a string or
+ *     UNDEFINED value (and default does not evaluate to a non-empty string).
+ *   - otherwise, UNDEFINED if the first argument evaluates to UNDEFINED
+ *   - otherwise, a string specifying the user's home directory.
+ */
+
+static bool
+userHome_func(const char *                 name,
+	const classad::ArgumentList &arguments,
+	classad::EvalState          &state,
+	classad::Value              &result)
+{
+	if ((arguments.size() != 1) && (arguments.size() != 2))
+	{
+		std::stringstream ss;
+		result.SetErrorValue();
+		ss << "Invalid number of arguments passed to " << name << "; " << arguments.size() << "given, 1 required and 1 optional.";
+		classad::CondorErrMsg = ss.str();
+		return false;
+	}
+
+#ifdef WIN32
+	result.SetErrorValue();
+	classad::CondorErrMsg = "UserHome is not available on Windows.";
+	return true;
+#endif
+
+	std::string default_home;
+	classad::Value default_home_value;
+	if (arguments.size() != 2 || !arguments[1]->Evaluate(state, default_home_value) || !default_home_value.IsStringValue(default_home))
+	{
+		default_home = "";
+	}
+
+	std::string owner_string;
+	classad::Value owner_value;
+	if (!arguments[0]->Evaluate(state, owner_value))
+	{
+		
+	}
+	if (owner_value.IsUndefinedValue() && !default_home.size())
+	{
+		result.SetUndefinedValue();
+		return true;
+	}
+	if (!owner_value.IsStringValue(owner_string))
+	{
+		std::string unp_string;
+		std::stringstream ss;
+		classad::ClassAdUnParser unp; unp.Unparse(unp_string, arguments[0]);
+		ss << "Could not evaluate the first argument of " << name << " to string.  Expression: " << unp_string << ".";
+		return return_home_result(default_home, ss.str(), result, true);
+	}
+
+	errno = 0;
+#ifndef WIN32
+		// The UserHome function has potential side-effects (such as hitting a LDAP server)
+		// that might have a noticable impact on the system.  Disable unless explicitly requested.
+		// (really, this only is useful in the very limited HTCondor-CE context.)
+	if (!param_boolean("CLASSAD_ENABLE_USER_HOME", false ))
+	{
+		return return_home_result(default_home, "UserHome is currently disabled; to enable set CLASSAD_ENABLE_USER_HOME=true in the HTCondor config.",
+			result, false);
+	}
+	struct passwd *info = getpwnam(owner_string.c_str());
+	if (!info)
+	{
+		std::stringstream ss;
+		ss << "Unable to find home directory for user " << owner_string;
+		if (errno) {
+			ss << ": " << strerror(errno) << "(errno=" << errno << ")";
+		} else {
+			ss << ": No such user.";
+		}
+		return return_home_result(default_home, ss.str(), result, false);
+	}
+
+	if (!info->pw_dir)
+	{
+		std::stringstream ss;
+		ss << "User " << owner_string << " has no home directory.";
+		return return_home_result(default_home, ss.str(), result, false);
+	}
+	std::string home_string = info->pw_dir;
+	result.SetStringValue(home_string);
+#endif
+
+	return true;
+}
+
+
 static
-void registerStrlistFunctions()
+void registerClassadFunctions()
 {
 	std::string name;
+	name = "envV1ToV2";
+	classad::FunctionCall::RegisterFunction(name, EnvironmentV1ToV2);
+
+	name = "mergeEnvironment";
+	classad::FunctionCall::RegisterFunction(name, MergeEnvironment);
+
+	name = "listToArgs";
+	classad::FunctionCall::RegisterFunction(name, ListToArgs);
+
+	name = "argsToList";
+	classad::FunctionCall::RegisterFunction(name, ArgsToList);
+
 	name = "stringListSize";
 	classad::FunctionCall::RegisterFunction( name,
 											 stringListSize_func );
@@ -588,6 +1060,9 @@ void registerStrlistFunctions()
 	classad::FunctionCall::RegisterFunction( name,
 											 stringListRegexpMember_func );
 
+	name = "userHome";
+	classad::FunctionCall::RegisterFunction(name, userHome_func);
+
 	// user@domain, slot@machine & sinful string crackers.
 	name = "splitusername";
 	classad::FunctionCall::RegisterFunction( name, splitAt_func );
@@ -608,19 +1083,9 @@ ClassAd::ClassAd()
 {
 	if ( !m_initConfig ) {
 		this->Reconfig();
-		registerStrlistFunctions();
-		classad::ExprTree::set_user_debug_function(classad_debug_dprintf);
 		m_initConfig = true;
 	}
 
-		// Compatibility ads are born with this to emulate the special
-		// CurrentTime in old ClassAds. We don't protect it afterwards,
-		// but that shouldn't be problem unless someone is deliberately
-		// trying to shoot themselves in the foot.
-	if ( !m_strictEvaluation ) {
-		AssignExpr( ATTR_CURRENT_TIME, "time()" );
-	}
-
 	ResetName();
     ResetExpr();
 
@@ -631,20 +1096,11 @@ ClassAd::ClassAd( const ClassAd &ad ) : classad::ClassAd(ad)
 {
 	if ( !m_initConfig ) {
 		this->Reconfig();
-		registerStrlistFunctions();
 		m_initConfig = true;
 	}
 
 	CopyFrom( ad );
 
-		// Compatibility ads are born with this to emulate the special
-		// CurrentTime in old ClassAds. We don't protect it afterwards,
-		// but that shouldn't be problem unless someone is deliberately
-		// trying to shoot themselves in the foot.
-	if ( !m_strictEvaluation ) {
-		AssignExpr( ATTR_CURRENT_TIME, "time()" );
-	}
-
 	ResetName();
     ResetExpr();
 
@@ -654,20 +1110,11 @@ ClassAd::ClassAd( const classad::ClassAd &ad )
 {
 	if ( !m_initConfig ) {
 		this->Reconfig();
-		registerStrlistFunctions();
 		m_initConfig = true;
 	}
 
 	CopyFrom( ad );
 
-		// Compatibility ads are born with this to emulate the special
-		// CurrentTime in old ClassAds. We don't protect it afterwards,
-		// but that shouldn't be problem unless someone is deliberately
-		// trying to shoot themselves in the foot.
-	if ( !m_strictEvaluation ) {
-		AssignExpr( ATTR_CURRENT_TIME, "time()" );
-	}
-
 	ResetName();
     ResetExpr();
 
@@ -682,17 +1129,10 @@ ClassAd( FILE *file, const char *delimitor, int &isEOF, int&error, int &empty )
 {
 	if ( !m_initConfig ) {
 		this->Reconfig();
-		registerStrlistFunctions();
 		m_initConfig = true;
 	}
 
-		// Compatibility ads are born with this to emulate the special
-		// CurrentTime in old ClassAds. We don't protect it afterwards,
-		// but that shouldn't be problem unless someone is deliberately
-		// trying to shoot themselves in the foot.
-	if ( !m_strictEvaluation ) {
-		AssignExpr( ATTR_CURRENT_TIME, "time()" );
-	}
+	DisableDirtyTracking();
 
 	ResetName();
     ResetExpr();
@@ -915,8 +1355,6 @@ int ClassAd::Insert( const char *name, classad::ExprTree *& expr, bool bCache )
 int
 ClassAd::Insert( const char *str )
 {
-	classad::ClassAdParser parser;
-
 		// String escaping is different between new and old ClassAds.
 		// We need to convert the escaping from old to new style before
 		// handing the expression to the new ClassAds parser.
@@ -1187,11 +1625,9 @@ EvalAttr( const char *name, classad::ClassAd *target, classad::Value & value)
 	int rc = 0;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttr( name, value ) ) {
 			rc = 1;
 		}
-		releaseTheMyRef( this );
 		return rc;
 	}
 
@@ -1216,12 +1652,10 @@ EvalString( const char *name, classad::ClassAd *target, char *value )
 	string strVal;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttrString( name, strVal ) ) {
 			strcpy( value, strVal.c_str( ) );
 			rc = 1;
 		}
-		releaseTheMyRef( this );
 		return rc;
 	}
 
@@ -1253,7 +1687,6 @@ EvalString (const char *name, classad::ClassAd *target, char **value)
 	int rc = 0;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttrString( name, strVal ) ) {
 
             *value = (char *)malloc(strlen(strVal.c_str()) + 1);
@@ -1264,7 +1697,6 @@ EvalString (const char *name, classad::ClassAd *target, char **value)
                 rc = 0;
             }
 		}
-		releaseTheMyRef( this );
 		return rc;
 	}
 
@@ -1325,11 +1757,9 @@ EvalInteger (const char *name, classad::ClassAd *target, long long &value)
 	classad::Value val;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttr( name, val ) ) { 
 			rc = 1;
 		}
-		releaseTheMyRef( this );
 	}
 	else 
 	{
@@ -1384,7 +1814,6 @@ EvalFloat (const char *name, classad::ClassAd *target, double &value)
 	bool boolVal;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttr( name, val ) ) {
 			if( val.IsRealValue( doubleVal ) ) {
 				value = doubleVal;
@@ -1399,7 +1828,6 @@ EvalFloat (const char *name, classad::ClassAd *target, double &value)
 				rc = 1;
 			}
 		}
-		releaseTheMyRef( this );
 		return rc;
 	}
 
@@ -1451,7 +1879,6 @@ EvalBool  (const char *name, classad::ClassAd *target, int &value)
 	bool boolVal;
 
 	if( target == this || target == NULL ) {
-		getTheMyRef( this );
 		if( EvaluateAttr( name, val ) ) {
 			if( val.IsBooleanValue( boolVal ) ) {
 				value = boolVal ? 1 : 0;
@@ -1464,7 +1891,6 @@ EvalBool  (const char *name, classad::ClassAd *target, int &value)
 				rc = 1;
 			}
 		}
-		releaseTheMyRef( this );
 		return rc;
 	}
 
@@ -1513,12 +1939,6 @@ initFromString( char const *str,MyString *err_msg )
 	// First, clear our ad so we start with a fresh ClassAd
 	Clear();
 
-		// Reinsert CurrentTime, emulating the special version in old
-		// ClassAds
-	if ( !m_strictEvaluation ) {
-		AssignExpr( ATTR_CURRENT_TIME, "time()" );
-	}
-
 	char *exprbuf = new char[strlen(str)+1];
 	ASSERT( exprbuf );
 
@@ -1567,13 +1987,15 @@ fPrintAd( FILE *file, const classad::ClassAd &ad, bool exclude_private, StringLi
 }
 
 void
-dPrintAd( int level, const classad::ClassAd &ad )
+dPrintAd( int level, const classad::ClassAd &ad, bool exclude_private )
 {
-	MyString buffer;
+	if ( IsDebugCatAndVerbosity( level ) ) {
+		MyString buffer;
 
-	sPrintAd( buffer, ad, true );
+		sPrintAd( buffer, ad, exclude_private );
 
-	dprintf( level|D_NOHEADER, "%s", buffer.Value() );
+		dprintf( level|D_NOHEADER, "%s", buffer.Value() );
+	}
 }
 
 int
@@ -1929,7 +2351,7 @@ SetDirtyFlag(const char *name, bool dirty)
 }
 
 void ClassAd::
-GetDirtyFlag(const char *name, bool *exists, bool *dirty)
+GetDirtyFlag(const char *name, bool *exists, bool *dirty) const
 {
 	if ( Lookup( name ) == NULL ) {
 		if ( exists ) {
@@ -2083,8 +2505,8 @@ void ClassAd::ChainCollapse()
 
 void ClassAd::
 GetReferences(const char* attr,
-                StringList &internal_refs,
-                StringList &external_refs)
+                StringList *internal_refs,
+                StringList *external_refs) const
 {
     ExprTree *tree;
 
@@ -2097,8 +2519,8 @@ GetReferences(const char* attr,
 
 bool ClassAd::
 GetExprReferences(const char* expr,
-				  StringList &internal_refs,
-				  StringList &external_refs)
+				  StringList *internal_refs,
+				  StringList *external_refs) const
 {
 	classad::ClassAdParser par;
 	classad::ExprTree *tree = NULL;
@@ -2143,8 +2565,8 @@ static void AppendReference( StringList &reflist, char const *name )
 
 void ClassAd::
 _GetReferences(classad::ExprTree *tree,
-			   StringList &internal_refs,
-			   StringList &external_refs)
+			   StringList *internal_refs,
+			   StringList *external_refs) const
 {
 	if ( tree == NULL ) {
 		return;
@@ -2154,15 +2576,13 @@ _GetReferences(classad::ExprTree *tree,
 	classad::References int_refs_set;
 	classad::References::iterator set_itr;
 
-	getTheMyRef( this );
 	bool ok = true;
-	if( !GetExternalReferences(tree, ext_refs_set, true) ) {
+	if( external_refs && !GetExternalReferences(tree, ext_refs_set, true) ) {
 		ok = false;
 	}
-	if( !GetInternalReferences(tree, int_refs_set, true) ) {
+	if( internal_refs && !GetInternalReferences(tree, int_refs_set, true) ) {
 		ok = false;
 	}
-	releaseTheMyRef( this );
 	if( !ok ) {
 		dprintf(D_FULLDEBUG,"warning: failed to get all attribute references in ClassAd (perhaps caused by circular reference).\n");
 		dPrintAd(D_FULLDEBUG, *this);
@@ -2175,32 +2595,33 @@ _GetReferences(classad::ExprTree *tree,
 		// StringLists.  This scales better than trying to remove
 		// duplicates while inserting into the StringList.
 
-	for ( set_itr = ext_refs_set.begin(); set_itr != ext_refs_set.end();
-		  set_itr++ ) {
-		const char *name = set_itr->c_str();
+	if ( external_refs ) {
+		for ( set_itr = ext_refs_set.begin(); set_itr != ext_refs_set.end();
+			  set_itr++ ) {
+			const char *name = set_itr->c_str();
 			// Check for references to things in the MatchClassAd
 			// and do the right thing.  This does not cover all
 			// possible ways of referencing things in the match ad,
 			// but it covers the ones users are expected to use
 			// and the ones expected from OptimizeAdForMatchmaking.
-		if ( strncasecmp( name, "target.", 7 ) == 0 ) {
-			AppendReference( external_refs, &set_itr->c_str()[7] );
-		} else if ( strncasecmp( name, "other.", 6 ) == 0 ) {
-			AppendReference( external_refs, &set_itr->c_str()[6] );
-		} else if ( strncasecmp( name, ".left.", 6 ) == 0 ) {
-			AppendReference( external_refs, &set_itr->c_str()[6] );
-		} else if ( strncasecmp( name, ".right.", 7 ) == 0 ) {
-			AppendReference( external_refs, &set_itr->c_str()[7] );
-		} else {
-			AppendReference( external_refs, set_itr->c_str() );
+			if ( strncasecmp( name, "target.", 7 ) == 0 ) {
+				AppendReference( *external_refs, &set_itr->c_str()[7] );
+			} else if ( strncasecmp( name, "other.", 6 ) == 0 ) {
+				AppendReference( *external_refs, &set_itr->c_str()[6] );
+			} else if ( strncasecmp( name, ".left.", 6 ) == 0 ) {
+				AppendReference( *external_refs, &set_itr->c_str()[6] );
+			} else if ( strncasecmp( name, ".right.", 7 ) == 0 ) {
+				AppendReference( *external_refs, &set_itr->c_str()[7] );
+			} else {
+				AppendReference( *external_refs, set_itr->c_str() );
+			}
 		}
 	}
 
-	for ( set_itr = int_refs_set.begin(); set_itr != int_refs_set.end();
-		  set_itr++ ) {
-		const char *name = set_itr->c_str();
-		if ( strncasecmp( name, "my.", 3 ) ) {
-			AppendReference( internal_refs, set_itr->c_str() );
+	if ( internal_refs ) {
+		for ( set_itr = int_refs_set.begin(); set_itr != int_refs_set.end();
+			  set_itr++ ) {
+			AppendReference( *internal_refs, set_itr->c_str() );
 		}
 	}
 }
@@ -2710,8 +3131,6 @@ static void InitTargetAttrLists()
 		machine_attrs_ad.AssignExpr( *attr, "True" );
 	}
 
-	machine_attrs_ad.Delete( ATTR_CURRENT_TIME );
-
 	tmp = param( "STARTD_EXPRS" );
 	if ( tmp ) {
 		tmp_strlist.initializeFromString( tmp );
@@ -2756,8 +3175,6 @@ static void InitTargetAttrLists()
 		job_attrs_ad.AssignExpr( *attr, "True" );
 	}
 
-	job_attrs_ad.Delete( ATTR_CURRENT_TIME );
-
 	tmp = param( "SUBMIT_EXPRS" );
 	if ( tmp ) {
 		tmp_strlist.initializeFromString( tmp );
@@ -2788,8 +3205,6 @@ static void InitTargetAttrLists()
 		schedd_attrs_ad.AssignExpr( *attr, "True" );
 	}
 
-	schedd_attrs_ad.Delete( ATTR_CURRENT_TIME );
-
 	target_attrs_init = true;
 }
 #endif
